#region Copyright Syncfusion Inc. 2001 - 2019
// Copyright Syncfusion Inc. 2001 - 2019. All rights reserved.
// Use of this code is subject to the terms of our license.
// A copy of the current license can be obtained at any time by e-mailing
// licensing@syncfusion.com. Any infringement will be prosecuted under
// applicable laws. 
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Syncfusion.Windows.Forms.Gauge;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace CustomRenderer_2008
{
  
    /// <summary>
    /// Provide information about custom renderer of Linear Gauage
    /// </summary>
    class CustomRenderer : ILinearGaugeRenderer
    {
        #region Varaibles
        /// </summary>
        /// Gets the Linear gauge
        /// </summary>
        private LinearGauge m_LinearGauge;
        /// <summary>
        /// Gets/Sets the Tick Distance of the Linear gauge.
        /// </summary>
        private float majorTicksDistance;
        /// <summary>
        /// Calculates the Minor Ticks Pixels.
        /// </summary>
        private float m_minorTicksPixels;
        /// <summary>
        /// Start point of the frame
        /// </summary>
        private int startpoint;
        /// <summary>
        /// Counts the Major ticks count for the given range.
        /// </summary>
        private int majorTicksCount;

        private Graphics line;

        #endregion

        #region Properties
        /// <summary>
        ///  Gets the Radial gauge
        /// </summary>
        internal LinearGauge LinearGauge
        {
            get
            {
                return m_LinearGauge;
            }
        }
        #endregion

        #region Constructor
        public CustomRenderer(LinearGauge linearGauge)
        {
            m_LinearGauge = linearGauge;
            majorTicksDistance = 0;
            m_minorTicksPixels = 0;
            startpoint = 10;
        }
        #endregion

        #region ILinearGaugeRenderer interface Methods
        /// <summary>
        /// Helps to draw frame
        /// </summary>
        /// <param name="g"></param>
        public void DrawFrame(Graphics g)
        {

        }

        /// <summary>
        /// Helps to draw value and outer region of the ranges
        /// </summary>
        /// <param name="Graphics"></param>
        public void DrawLines(Graphics Graphics)
        {
            line = Graphics;
            Pen majorTickPen = new Pen(LinearGauge.MajorTickMarkColor);
            Pen minorTickPen = new Pen(LinearGauge.MinorTickMarkColor);
            Brush brush = new SolidBrush(LinearGauge.ForeColor);
            StringFormat sf = new StringFormat();
            Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            Graphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
            majorTicksDistance = ((LinearGauge.MaximumValue - LinearGauge.MinimumValue) / LinearGauge.MajorDifference);
            majorTicksCount = ((int)(LinearGauge.MaximumValue - LinearGauge.MinimumValue) / (LinearGauge.MajorDifference)) + 1;
            double majortickValue = LinearGauge.MinimumValue;
            float tickPosition = 40f;
            float temp1 = 0;
            float s = (LinearGauge.MaximumValue - LinearGauge.MinimumValue) % LinearGauge.MajorDifference;
            sf.Alignment = StringAlignment.Center;
            sf.LineAlignment = StringAlignment.Center;
            float minortickValue = 0;
            float tickPositionminor = 0;
            GraphicsPath path = new GraphicsPath();
            int minorcount = LinearGauge.MinorTickCount;
            m_minorTicksPixels = ((this.LinearGauge.Height - 50) / majorTicksDistance);
            int x = (this.LinearGauge.Width - 24) / 2;

            int x1 = ((this.LinearGauge.Width - 24) / 2) + 24;
            temp1 = 0;
            for (int L = 1; L <= majorTicksCount; L++)
            {
                // Draw left major ticks
                Graphics.DrawLine(majorTickPen, x , this.LinearGauge.Height - tickPosition, x - LinearGauge.MajorTicksHeight , this.LinearGauge.Height - tickPosition);

                // Draw Right major ticks
                Graphics.DrawLine(majorTickPen, x1, this.LinearGauge.Height - tickPosition, x1 + LinearGauge.MajorTicksHeight, this.LinearGauge.Height - tickPosition);

                Graphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
                if (LinearGauge.ShowScaleLabel)
                {
                    // Draw left range value
                    Graphics.DrawString(string.Format("{0}C", Math.Round(majortickValue, 2).ToString()),
                             LinearGauge.Font, brush, new PointF(x - LinearGauge.MajorTicksHeight - 25, this.LinearGauge.Height - tickPosition), sf);
                  
                    // Draw right range value
                    Graphics.DrawString(Math.Round(majortickValue, 2).ToString(),
                            LinearGauge.Font, brush, new PointF(x1 + LinearGauge.MajorTicksHeight + 25, this.LinearGauge.Height - tickPosition), sf);
                }
                if (L == majorTicksCount)
                    minorcount = (LinearGauge.MinorTickCount * (int)Math.Ceiling(s)) / LinearGauge.MajorDifference;
                if (majortickValue < LinearGauge.MaximumValue)
                {
                    for (int S = 1; S <= minorcount; S++)
                    {
                        minortickValue = (m_minorTicksPixels / (LinearGauge.MinorTickCount + 1)) * S;
                        tickPositionminor = this.LinearGauge.Height - (minortickValue + temp1 + 40);

                        // Draw left major ticks
                        Graphics.DrawLine(minorTickPen, x, (float)tickPositionminor, x - LinearGauge.MinorTickHeight, (float)tickPositionminor);

                        // Draw Right major ticks
                        Graphics.DrawLine(minorTickPen, x1, (float)tickPositionminor, x1 + LinearGauge.MinorTickHeight, (float)tickPositionminor);

                    }
                    temp1 = m_minorTicksPixels * L;
                }

                majortickValue += LinearGauge.MajorDifference;
                tickPosition += m_minorTicksPixels;
            }
            minorTickPen.Dispose();
        }

        /// <summary>
        /// Helps to draw Ranges in linear gauge
        /// </summary>
        /// <param name="Graphics"></param>
        public void DrawRanges(Graphics Graphics)
        {
            Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
            foreach (LinearRange ptrRange in this.LinearGauge.Ranges)
            {
                int rvalve = (int)Math.Ceiling(LinearGauge.MaximumValue - ptrRange.EndValue) / LinearGauge.MajorDifference;
                if (ptrRange.EndValue > ptrRange.StartValue && ptrRange.EndValue <= this.LinearGauge.MaximumValue)
                {
                    if (this.LinearGauge.MinimumValue >= 0 && ptrRange.StartValue < 0)
                    {
                        return;
                    }
                    float startValue = (float)ptrRange.StartValue;
                    float endValue = (float)ptrRange.EndValue;
                    if (this.LinearGauge.MinimumValue < 0)
                    {
                        startValue = this.LinearGauge.MinimumValue + Math.Abs(ptrRange.StartValue);
                    }
                    if (this.LinearGauge.MinimumValue < 0 && ptrRange.StartValue > 0)
                    {
                        startValue = Math.Abs(this.LinearGauge.MinimumValue) + Math.Abs(ptrRange.StartValue);
                    }
                    if (this.LinearGauge.MinimumValue < 0 && ptrRange.StartValue == 0)
                    {
                        startValue = Math.Abs(this.LinearGauge.MinimumValue) + Math.Abs(ptrRange.StartValue);
                        startValue = (((startValue / LinearGauge.MajorDifference)) * m_minorTicksPixels);
                    }

                    float rangeheight = (ptrRange.EndValue / LinearGauge.MajorDifference) * m_minorTicksPixels;
                    float endvalueRangeHeight = 0f;
                    if (this.LinearGauge.MinimumValue < 0)
                    {
                        rangeheight = ((Math.Abs(this.LinearGauge.MinimumValue) + ptrRange.EndValue) / LinearGauge.MajorDifference) * m_minorTicksPixels;
                    }
                    endvalueRangeHeight = rangeheight;
                    if (this.LinearGauge.MinimumValue < 0 && ptrRange.StartValue == 0)
                    {
                        endvalueRangeHeight = (((ptrRange.EndValue - ptrRange.StartValue) / LinearGauge.MajorDifference) * m_minorTicksPixels);
                    }
                   
                    RectangleF outerRect = new RectangleF((this.LinearGauge.Width - 24) / 2, startpoint - 10 + (majorTicksDistance * m_minorTicksPixels) - rangeheight, 24, endvalueRangeHeight + 13);

                    PointF[] points = { new PointF(outerRect.Left, outerRect.Bottom), new PointF(outerRect.Left, outerRect.Top), new PointF(outerRect.Right, outerRect.Top), new PointF(outerRect.Right, outerRect.Bottom) };

                    Graphics.FillRectangle(new SolidBrush(ptrRange.Color), (outerRect.X + ((outerRect.Width - 8) / 2)), startpoint + (majorTicksDistance * m_minorTicksPixels) - rangeheight, 8, endvalueRangeHeight + 10);

                    // Draw outer arc
                    Graphics.DrawArc(new Pen(LinearGauge.GaugeBaseColor, 1), outerRect.Left - 8, outerRect.Bottom - 5, 40F, 40F, 310, 285);

                    // Draw outer Line
                    Graphics.DrawLines(new Pen(LinearGauge.GaugeBaseColor), points);

                    RectangleF innerRect = new RectangleF((outerRect.X + ((outerRect.Width - 8) / 2)), startpoint + (majorTicksDistance * m_minorTicksPixels) - ((((Math.Abs(this.LinearGauge.MinimumValue) + LinearGauge.Value) / LinearGauge.MajorDifference)) * m_minorTicksPixels), 8, ((((Math.Abs(this.LinearGauge.MinimumValue) + LinearGauge.Value) / LinearGauge.MajorDifference)) * m_minorTicksPixels) + 2);

                    // Draw inner rectangle
                    Graphics.FillRectangle(new SolidBrush(LinearGauge.ValueIndicatorColor), innerRect.Left, startpoint + (majorTicksDistance * m_minorTicksPixels) - ((((Math.Abs(this.LinearGauge.MinimumValue) + LinearGauge.Value) / LinearGauge.MajorDifference)) * m_minorTicksPixels), 8, ((((Math.Abs(this.LinearGauge.MinimumValue) + LinearGauge.Value) / LinearGauge.MajorDifference)) * m_minorTicksPixels) + 8);
                    // Draw inner arc
                    Graphics.FillEllipse(new SolidBrush(LinearGauge.ValueIndicatorColor), innerRect.X - 11, innerRect.Bottom, 30F, 30F);
                }
            }
        }

        public void DrawPointer(Graphics Graphics)
        {

        }

        public void UpdateRenderer(System.Windows.Forms.PaintEventArgs PaintEventArgs)
        {
            DrawFrame(PaintEventArgs.Graphics);
            DrawLines(PaintEventArgs.Graphics);
            DrawRanges(PaintEventArgs.Graphics);
            DrawPointer(PaintEventArgs.Graphics);
        }

        #endregion
    }


}
